#include <DMD_STM32.h>
#include <TimeLib.h>
#include "UARTTransfer.h"
#include <RTClock.h>
#include <Wire.h>
#include <BH1750.h>
// big digit font
#include "mambuat.h"

BH1750 lightMeter;
RTClock rt (RTCSEL_LSE); // initialise

// message id's for STM32 <-> ESP communication
#define SET_NTP_TIME 51
#define GET_NTP_TIME 52

// check NTP time every
#define NTP_CHECK_INT (1000*3600*6)  // six hours

// DMD matrix connect to SPI_2 
SPIClass dmd_spi2(2);

// number of DMD matrices
#define DISPLAYS_ACROSS 2
#define DISPLAYS_DOWN 1

// ----- Select pins for P10 matrix connection ------------
// pins A, B, SCLK may be any digital I/O, pin nOE should be PWM pin as PB1,PA8

// SPI specific pins as CLK and R_DATA has predefined values:
//   for SPI(1) CLK = PA5  R_DATA = PA7
//   for SPI(2) CLK = PB13  R_DATA = PB15
// --------------------------------------------------------
#define DMD_PIN_A PB11
#define DMD_PIN_B PB12
#define DMD_PIN_nOE PB1
#define DMD_PIN_SCLK PB10

//Fire up the DMD library at first as dmd
DMD dmd(DMD_PIN_A, DMD_PIN_B, DMD_PIN_nOE, DMD_PIN_SCLK, DISPLAYS_ACROSS, DISPLAYS_DOWN, dmd_spi2 );

DMD_GFX_Font Mambuat((uint8_t*)&mambuat, 0);

uint32_t tt;
uint32_t prev_ntp_check = 0;
UARTTransfer UT;

// table of brightness levels 
#define NUM_LEVELS 5
uint16_t luxlevels[NUM_LEVELS] = {10, 50, 100, 500, 2000};
uint8_t brightlevels[NUM_LEVELS] = {3, 25, 40, 100, 150};

/*--------------------------------------------------------------------------------------
  setup
  Called by the Arduino architecture before the main loop begins
  --------------------------------------------------------------------------------------*/
void setup(void)
{ 
  Wire.begin();
  lightMeter.begin();
  dmd.init();
  dmd.clearScreen( true );
  dmd.setBrightness(3);
  Serial2.begin(9600);
  UT.begin(&Serial2);
  tt = rt.getTime();
  dmd.selectFont(&Mambuat);
  DisplayTime(tt);
 }

void BrightCheck() {
  static float cur_bright = 10;
  static uint32_t prev_check = 0;
  uint8_t new_bright;
  float Kb = 0.9;

  if (( millis() - prev_check) < 1000) return;
  prev_check = millis();
  uint32_t lux = lightMeter.readLightLevel();
  if (lux <= luxlevels[0]) new_bright = brightlevels[0];
  else {
    uint8_t i = 0;
    while ((lux > luxlevels[i]) && (i < NUM_LEVELS)) i++;
    if (lux > luxlevels[i]) new_bright = brightlevels[i];
    else {
      new_bright = map(lux, luxlevels[i - 1], luxlevels[i], brightlevels[i - 1], brightlevels[i]);
    }
  }
  cur_bright = (float)(cur_bright * Kb + new_bright * (1 - Kb));
  dmd.setBrightness((uint8_t)cur_bright);
}

void ColonBlink(uint32_t time) {
  uint8_t colon_on;
  static uint32_t ttt = 0;
  if (ttt == time) return;
  ttt = time;
  colon_on = (ttt%2)? GRAPHICS_OR : GRAPHICS_NOR;
  for (uint8_t k =0; k < DISPLAYS_ACROSS; k++) {
    dmd.drawChar( 15 + k*32,  4, ':', colon_on     ); 
  }
}
void DisplayTime(uint32_t time) {
  static uint8_t prev_h = 0;
  static uint8_t prev_m = 0;
  uint8_t h = hour(time);
  uint8_t m = minute(time);
  if ((h == prev_h) && (m == prev_m)) return;
  prev_h = h;
  prev_m = m;
  uint8_t h1 = h / 10;
  uint8_t h2 = h % 10;
  uint8_t m1 = m / 10;
  uint8_t m2 = m % 10;
  for (uint8_t k =0; k < DISPLAYS_ACROSS; k++) {
  uint8_t offset = k*32;
  dmd.drawChar(  1 + offset,  3, '0' + h1, GRAPHICS_NORMAL ); // thousands
  dmd.drawChar(  8 + offset,  3, '0' + h2,  GRAPHICS_NORMAL ); // hundreds
  //dmd.drawChar( 15 + offset,  4, ':', GRAPHICS_OR     ); // clock colon overlay on
  dmd.drawChar( 18 + offset,  3, '0' + m1,   GRAPHICS_NORMAL ); // tens
  dmd.drawChar( 25 + offset,  3, '0' + m2,          GRAPHICS_NORMAL ); // units
  }
}
void loop() {

  if (UT.receiveData()) {
    uint8_t m_id = UT.getMessageID();
    if ((m_id == SET_NTP_TIME) && (UT.getDataLength() ==  4 )) {
      UT.getData((uint8_t*)&tt, sizeof(tt));
      rt.setTime(tt);
      prev_ntp_check = millis();
    }
    UT.clearMessage();
  }
  
  if (millis() - prev_ntp_check > NTP_CHECK_INT ) {
     UT.sendData(GET_NTP_TIME, 4, (uint8_t*) &tt);
     prev_ntp_check = millis();
  }
  
  tt = rt.getTime();
  DisplayTime(tt);
  ColonBlink(tt);
  BrightCheck();
  delay(1000);
}
